@@ -1,20 +1,17 @@
 from django.views.generic import ListView
+from django.db.models import Q
+from django.shortcuts import redirect
+from django.template.defaultfilters import slugify
+
 from snowebsvg.models import Collection, GroupSvg, Svg
 
 
-class AppMixin(ListView):
-    def post(self, request, *args, **kwargs):
-        if 'theme' in request.POST:
-            request.session['theme'] = request.POST['theme']
-        return super().get(request, *args, **kwargs)
-
-
-class CollectionList(AppMixin):
+class CollectionList(ListView):
     model = Collection
     template_name = 'app/collection_list.html'
 
 
-class GroupSvgList(AppMixin):
+class GroupSvgList(ListView):
     model = GroupSvg
     template_name = 'app/groupsvg_list.html'
 
@@ -25,7 +22,7 @@ class GroupSvgList(AppMixin):
         return self.model.objects.all()
 
 
-class SvgList(AppMixin):
+class SvgList(ListView):
     model = Svg
     template_name = 'app/svg_list.html'
 
@@ -33,13 +30,6 @@ class SvgList(AppMixin):
         group_key = self.kwargs.get('group_key')
         collection_key = self.kwargs.get('collection_key')
         svg_key = self.kwargs.get('svg_key')
-        if group_key and not collection_key and not svg_key:
-            return self.model.objects.filter(group__key=group_key)
-        if group_key and collection_key and not svg_key:
-            return self.model.objects.filter(
-                group__key=group_key,
-                group__collection__key=collection_key
-            )
         if group_key and collection_key and svg_key:
             return self.model.objects.filter(
                 group__key=group_key,
@@ -47,3 +37,31 @@ class SvgList(AppMixin):
                 key=svg_key
             )
         return self.model.objects.all()
+
+
+class SvgSearch(ListView):
+    model = Svg
+    template_name = 'app/svg_search.html'
+
+    def get_queryset(self):
+        key = self.kwargs.get('key')
+        if key:
+            return self.model.objects.filter(
+                Q(key__icontains=key) |
+                Q(group__key__icontains=key) |
+                Q(group__collection__key__icontains=key)
+            )
+        return self.model.objects.all()
+
+    def get_context_data(self, **kwargs):
+        context = super(SvgSearch, self).get_context_data(**kwargs)
+        context['key'] = self.kwargs.get('key')
+        return context
+
+    def post(self, request, *args, **kwargs):
+        key = request.POST.get('key', None)
+        if key:
+            key_slug = slugify(key)
+            if key_slug:
+                return redirect('app:svg_search', key=key_slug)
+        return super().post(request, *args, **kwargs)
